# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'foursight_core', 'helpers': 'foursight_core/checks/helpers'}

packages = \
['checks',
 'checks.helpers',
 'foursight_core',
 'foursight_core.checks',
 'foursight_core.checks.helpers',
 'foursight_core.react.api',
 'foursight_core.scripts',
 'foursight_core.scripts.save',
 'helpers']

package_data = \
{'': ['*'],
 'foursight_core': ['react/ui/*',
                    'react/ui/static/css/*',
                    'react/ui/static/js/*',
                    'templates/*']}

install_requires = \
['Jinja2==2.10.1',
 'MarkupSafe==1.1.1',
 'PyJWT==2.5.0',
 'click>=7.1.2,<8.0.0',
 'cron-descriptor>=1.2.31,<2.0.0',
 'dcicutils>=6.0.0,<7.0.0',
 'elasticsearch-dsl>=7.0.0,<8.0.0',
 'elasticsearch==7.13.4',
 'geocoder==1.38.1',
 'gitpython>=3.1.2,<4.0.0',
 'google-api-python-client>=1.12.5,<2.0.0',
 'google-auth-oauthlib>=0.7.0,<0.8.0',
 'google>=3.0.0,<4.0.0',
 'pyDes>=2.0.1,<3.0.0',
 'pytz>=2020.1,<2021.0']

entry_points = \
{'console_scripts': ['decrypt-accounts-file = '
                     'foursight_core.scripts.decrypt_accounts_file:main',
                     'encrypt-accounts-file = '
                     'foursight_core.scripts.encrypt_accounts_file:main']}

setup_kwargs = {
    'name': 'foursight-core',
    'version': '3.0.0.1b1',
    'description': 'Serverless Chalice Application for Monitoring',
    'long_description': 'None',
    'author': '4DN-DCIC Team',
    'author_email': 'support@4dnucleome.org',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<3.9',
}


setup(**setup_kwargs)
